# -*- coding: utf-8 -*-
## Filename    : TestMCLAnalyser.py
## Author(s)   : Michel Le Borgne
## Created     : 03/2012
## Revision    :
## Source      :
##
## Copyright 2012 : IRISA/IRSET
##
## This library is free software; you can redistribute it and/or modify it
## under the terms of the GNU General Public License as published
## by the Free Software Foundation; either version 2.1 of the License, or
## any later version.
##
## This library is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY, WITHOUT EVEN THE IMPLIED WARRANTY OF
## MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  The software and
## documentation provided here under is on an "as is" basis, and IRISA has
## no obligations to provide maintenance, support, updates, enhancements
## or modifications.
## In no event shall IRISA be liable to any party for direct, indirect,
## special, incidental or consequential damages, including lost profits,
## arising out of the use of this software and its documentation, even if
## IRISA have been advised of the possibility of such damage.  See
## the GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this library; if not, write to the Free Software Foundation,
## Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
##
## The original code contained here was initially developed by:
##
##     Michel Le Borgne.
##     IRISA
##     Symbiose team
##     IRISA  Campus de Beaulieu
##     35042 RENNES Cedex, FRANCE
##
##
## Contributor(s): Geoffroy Andrieux - IRISA/IRSET
##
"""
Unitary Tests for the MCLAnalyser
"""
from __future__ import print_function
import pkg_resources
import unittest
import os

from cadbiom.models.clause_constraints.mcl.MCLAnalyser import MCLAnalyser
from cadbiom.models.guard_transitions.simulator.chart_simul import ChartSimulator
from cadbiom.models.guard_transitions.simulator.simul_exceptions import SimulStopException
from cadbiom.models.guard_transitions.chart_model import ChartModel
from cadbiom.models.clause_constraints.mcl.MCLQuery import MCLSimpleQuery
from cadbiom.models.clause_constraints.mcl.MCLSolutions import FrontierSolution
from cadbiom.commons import DIR_LOGS

# helper fonctions for test automation
def lit_subsol_equal(ssol1, ssol2):
    """
    @param ssol1,ssol2: list<string>
    CAREFULL: change order in lists
    """
    print("subsol type ?", isinstance(ssol2, list))
    if not isinstance(ssol2, list):
        ssol2 = list(ssol2)
    print("SUBSOL", ssol1, ssol2)
    return ssol1 == ssol2

def lit_sol_equal(sol1, sol2):
    """
    @param sol1,sol2: list<list<string>>
    """
    if not isinstance(sol2, list):
        sol1 = tuple(tuple(ssol) for ssol in sol2)
    print("LIT_SOL", sol1, sol2)
    return sol1 == sol2

# simple reporter
class ErrorRep(object):
    """
    error reporter of the compil type
    """
    def __init__(self):
        self.context = ""
        self.error = False
        pass

    def display(self, mess):
        """
        just printing
        """
        self.error = True
        print('\n>> '+self.context+"  "+mess)

    def display_info(self, mess):
        """
        just printing
        """
        print('\n-- '+mess)

    def set_context(self, cont):
        """
        for transition compiling
        """
        self.context = cont



class TestMCLAnaLyzer(unittest.TestCase):
    """
    Unit tests for MCLAnalyser
    """

    def test_lit_sol_equal(self):
        """
        Test the test helper function
        """
        sol1 = [['-_lit2', '-C4', 'h2', '-A1', '-C3', '-A3', '-A2', '-A4'], \
                ['-_lit2', '-C4', '-h2', '-A1', '-C3', '-A3', '-A2', '-A4'],\
                ['-_lit2', '-C4', '-h2', '-A1', '-C3', '-A3', '-A2', '-A4']]
        sol2 = [['-_lit2', '-C4', 'h2', '-A1', '-C3', '-A3', '-A2', '-A4'], \
                ['-_lit2', '-C4', '-h2', '-A1', '-C3', '-A3', '-A2', '-A4'],\
                ['-_lit2', '-C4', '-h2', '-A1', '-C3', '-A3', '-A2', '-A4']]
        res = lit_sol_equal(sol1, sol2)
        self.assert_(res, 'Error in lit_sol equality')
        sol2 = [['-_lit2', '-C4', 'h2', '-A1', '-C3', '-A3', '-A2', '-A4'], \
                ['-_lit2', '-C4', '-h2', '-A1', '-C3', '-A3', '-A2', '-A4'],\
                ['-_lit2', 'C4', '-h2', '-A1', '-C3', '-A3', '-A2', '-A4']]
        res = lit_sol_equal(sol1, sol2)
        self.assert_(not res, 'Error in lit_sol inequality')
        # test on numbers
        sol1 = [[], [3], [], [5]]
        sol2 = [[], [3], [], [5]]
        res = lit_sol_equal(sol1, sol2)
        self.assert_(res, 'Error in lit_sol equality: number case')
        sol2 = [[], [3], [2], []]
        res = lit_sol_equal(sol1, sol2)
        self.assert_(not res, 'Error in lit_sol inequality: number case')

    @unittest.skip("Test files not provided")
    def test_load1(self):
        """
        Load from a bcx file
        """
        rep = ErrorRep()
        mcla = MCLAnalyser(rep)
        mcla.build_from_chart_file("../ucl/examples/test_tgfb_ref_300511.bcx")
        res = rep.error
        self.assert_(not res, 'Error in load1')

    def test_load2(self):
        """
        Load from a cadlang file
        """
        rep = ErrorRep()
        mcla = MCLAnalyser(rep)
        filename = pkg_resources.resource_filename(
            __name__, # package name
            "../../guard_transitions/translators/tests/tgf_cano_cmp.cal"
        )
        mcla.build_from_cadlang(filename)
        res = rep.error
        self.assert_(not res, 'Error in load2')

    @unittest.skip("Test files not provided")
    def test_load3(self):
        """
        Load from a PID file
        """
        rep = ErrorRep()
        mcla = MCLAnalyser(rep)
        mcla.build_from_pid_file("../ucl/examples/test_plk3_pathway.xml")
        res = rep.error
        self.assert_(not res, 'Error in load3')

    def test_delay1(self):
        """
        simple delay computation
        """
        rep = ErrorRep()
        mcla = MCLAnalyser(rep)
        filename = pkg_resources.resource_filename(
            __name__, # package name
            "examples/delay1.cal"
        )
        mcla.build_from_cadlang(filename)

        f_prop =  "C3 and not C4"
        query = MCLSimpleQuery(None, None, f_prop)

        lsol = mcla.sq_solutions(query,
                                 4, 1, mcla.unfolder.frontier_values)

#        if len(lsol) > 0:
#            print 'C3 ALONE'
#            print lsol[0] #first solution _ we have two
#        else:
#            print 'NO SOLUTION C4'

        res =  len(lsol) == 2
        expected_sol = [
            ['-_lit2', '-C4', 'h2', '-A1', '-C3', '-A3', '-A2', '-A4', '-B2', '-B3', '-_lit3', '-C2', 'C1', '_lit0', '-_lit1', 'B1'],
            ['-_lit2', '-C4', '-h2', '-A1', '-C3', '-A3', '-A2', '-A4', 'B2', '-B3', '-_lit3', 'C2', '-C1', '-_lit0', '-_lit1', '-B1'],
            ['-_lit2', '-C4', '-h2', '-A1', '-C3', '-A3', '-A2', '-A4', '-B2', 'B3', '_lit3', 'C2', '-C1', '-_lit0', '_lit1', '-B1']]
        res = res and lit_sol_equal(lsol[0].__str__(), expected_sol)
        self.assert_(not res, 'Error in C3 alone')

        f_prop = "C4 and not C3"
        query = MCLSimpleQuery(None, None, f_prop)

        lsol = mcla.sq_solutions(query,
                                 4, 1, mcla.unfolder.frontier_values)

#        if len(lsol)>0:
#            print 'C4 ALONE', len(lsol)
#            print lsol[0]
#        else:
#            print 'NO SOLUTION C4'

        res =  len(lsol) == 1
        expected_sol = [
            ['-_lit2', '-C4', '-h2', 'A1', '-C3', '-A3', '-A2', '-A4', '-B2', '-B3', '-_lit3', '-C2', 'C1', '-_lit0', '-_lit1', 'B1'],
            ['-_lit2', '-C4', '-h2', '-A1', '-C3', '-A3', 'A2', '-A4', '-B2', '-B3', '-_lit3', 'C2', '-C1', '-_lit0', '-_lit1', 'B1'],
            ['-_lit2', '-C4', '-h2', '-A1', '-C3', 'A3', '-A2', '-A4', '-B2', '-B3', '-_lit3', 'C2', '-C1', '-_lit0', '-_lit1', 'B1'],
            ['_lit2', '-C4', 'h2', '-A1', '-C3', '-A3', '-A2', 'A4', '-B2', '-B3', '_lit3', 'C2', '-C1', '_lit0', '-_lit1', 'B1']]
        res = res and lit_sol_equal(lsol[0].__str__(), expected_sol)
        self.assert_(not res, 'Error in C4 alone')

        f_prop = "C3 and C4"
        query = MCLSimpleQuery(None, None, f_prop)

        lsol = mcla.sq_solutions(query,
                                 4, 1, mcla.unfolder.frontier_values)

#        if len(lsol)>0:
#            print 'C3 and C4'
#            print lsol[0]
#        else:
#            print 'NO SOLUTION'

        res =  len(lsol) == 1
        expected_sol =  [
            ['-_lit2', '-C4', '-h2', 'A1', '-C3', '-A3', '-A2', '-A4', '-B2', '-B3', '-_lit3', '-C2', 'C1', '-_lit0', '-_lit1', 'B1'],
            ['-_lit2', '-C4', 'h2', '-A1', '-C3', '-A3', 'A2', '-A4', '-B2', '-B3', '-_lit3', 'C2', '-C1', '_lit0', '-_lit1', 'B1'],
            ['-_lit2', '-C4', '-h2', '-A1', '-C3', 'A3', '-A2', '-A4', 'B2', '-B3', '-_lit3', 'C2', '-C1', '-_lit0', '-_lit1', '-B1'],
            ['_lit2', '-C4', '-h2', '-A1', '-C3', '-A3', '-A2', 'A4', '-B2', 'B3', '_lit3', 'C2', '-C1', '-_lit0', '_lit1', '-B1']]
        res = res and lit_sol_equal(lsol[0].__str__(), expected_sol)
        self.assert_(not res, 'Error in C3 and C4')

    def test_delay1_w_ic(self):
        """
        simple delay computation with clock sequence

        First test: f_prop = "C4"
        # old solver
        [[], [], [3], []]
        [[-1, -2, -3, 4, -5, -6, -7, -8, -9, -10, -11, -12, 13, -14, -15, 16],
        [-1, -2, -3, -4, -5, -6, 7, -8, -9, -10, -11, 12, -13, -14, -15, 16],
        [-1, -2, 3, -4, -5, 6, -7, -8, -9, -10, -11, 12, -13, 14, -15, 16], #3>0 free clocks: (3, ':', 'h2')
        [1, -2, -3, -4, -5, -6, -7, 8, 9, -10, 11, 12, -13, -14, -15, -16]]
        ['A1', 'C1', 'B1']
        ['A2', 'C2', 'B1']
        ['h2', 'A3', 'C2', '_lit0', 'B1'] # activated clock: h2
        ['_lit2', 'A4', 'B2', '_lit3', 'C2']

        [[], [], [], []]
        [[-1, -2, -3, 4, -5, -6, -7, -8, -9, -10, -11, -12, 13, -14, -15, -16],
        [-1, -2, -3, -4, -5, -6, 7, -8, -9, -10, -11, 12, -13, -14, -15, -16],
        [-1, -2, -3, -4, -5, 6, -7, -8, -9, -10, -11, 12, -13, -14, -15, -16],
        [1, -2, -3, -4, -5, -6, -7, 8, -9, -10, 11, 12, -13, -14, -15, -16]]
        ['A1', 'C1']
        ['A2', 'C2']
        ['A3', 'C2']
        ['_lit2', 'A4', '_lit3', 'C2']


        # new solver (no clock used => better solution):
        [[], [], [], []]
        [[-1, -2, -3, 4, -5, -6, -7, -8, -9, -10, -11, -12, 13, -14, -15, -16],
        [-1, -2, -3, -4, -5, -6, 7, -8, -9, -10, -11, 12, -13, -14, -15, -16],
        [-1, -2, -3, -4, -5, 6, -7, -8, -9, -10, -11, 12, -13, -14, -15, -16],
        [1, -2, -3, -4, -5, -6, -7, 8, -9, -10, 11, 12, -13, -14, -15, -16]]
        ['A1', 'C1']
        ['A2', 'C2']
        ['A3', 'C2']
        ['_lit2', 'A4', '_lit3', 'C2']

        [[], [], [], []]
        [[-1, -2, -3, 4, -5, -6, -7, -8, -9, -10, -11, -12, 13, -14, -15, 16],
        [-1, -2, -3, -4, -5, -6, 7, -8, -9, -10, -11, 12, -13, -14, -15, 16],
        [-1, -2, -3, -4, -5, 6, -7, -8, -9, -10, -11, 12, -13, -14, -15, 16],
        [1, -2, -3, -4, -5, -6, -7, 8, -9, -10, 11, 12, -13, -14, -15, 16]]
        ['A1', 'C1', 'B1']
        ['A2', 'C2', 'B1']
        ['A3', 'C2', 'B1']
        ['_lit2', 'A4', '_lit3', 'C2', 'B1']

        .. note:: Debugging of solutions - help:

            for i, sol in enumerate(lsol, 1):
                print("Sol", i)
                # [[], [], [], []]
                print(sol.extract_act_input_clock_seq())
                # [[....], [....], [....], [....]]
                print(x.unflatten())

                # Convert literal values to str, display only activated ones (val > 0)
                for step in sol.unflatten():
                    print([sol.unfolder.get_var_name(val) for val in step
                           if val > 0])
        """

        def debug_solutions(lsol):
            """Make human readable the status of literals at each step of a sol
            """
            for i, sol in enumerate(lsol, 1):
                print("Sol", i)
                # [[], [], [], []]
                print(sol.extract_act_input_clock_seq())
                # [[....], [....], [....], [....]]
                print(sol.unflatten())

                # Convert literal values to str, display only activated ones (val > 0)
                for step in sol.unflatten():
                    print([sol.unfolder.get_var_name(val) for val in step
                           if val > 0])

        rep = ErrorRep()
        mcla = MCLAnalyser(rep)
        filename = pkg_resources.resource_filename(
            __name__, # package name
            "examples/delay1.cal"
        )
        mcla.build_from_cadlang(filename)

        st_prop = None
        inv_prop = None
        f_prop =  "C4"
        query = MCLSimpleQuery(st_prop, inv_prop, f_prop)
        # 4 steps, 1 solution
        lsol = mcla.sq_solutions(query, 4, 1, mcla.unfolder.frontier_values)
        sol = lsol[0]
        ic_seq = sol.extract_act_input_clock_seq()
        #print ic_seq
        expected_sol = [[], [], [], []]
        debug_solutions(lsol)
        res =  len(lsol) != 0
        res = res and lit_sol_equal(expected_sol, ic_seq)
        self.assert_(res, 'Error clock sequence for C4; expected: {}'.format(expected_sol))

        f_prop = "C4 and not C3"
        query = MCLSimpleQuery(None, None, f_prop)
        lsol = mcla.sq_solutions(query, 4, 1, mcla.unfolder.frontier_values)
        sol = lsol[0]
        ic_seq = sol.extract_act_input_clock_seq()
        #print ic_seq
        expected_sol = [[], [], [], []]
        debug_solutions(lsol)
        res =  len(lsol) != 0
        res = res and lit_sol_equal(expected_sol, ic_seq)
        self.assert_(res, 'Error clock sequence for C4 and not C3; expected: {}'.format(expected_sol))

        f_prop = "C3 and C4"
        query = MCLSimpleQuery(None, None, f_prop)
        lsol = mcla.sq_solutions(query, 4, 1, mcla.unfolder.frontier_values)
        sol = lsol[0]
        ic_seq = sol.extract_act_input_clock_seq()
        #print ic_seq
        # h2 is mandatory to get C3
        expected_sol = [[], [3], [], []]
        debug_solutions(lsol)
        res =  len(lsol) != 0
        res = res and lit_sol_equal(expected_sol, ic_seq)
        self.assert_(res, 'Error clock sequence for C4; expected: {}'.format(expected_sol))


    def test_input1(self):
        """
        simple input computation
        """
        rep = ErrorRep()
        mcla = MCLAnalyser(rep)
        filename = pkg_resources.resource_filename(
            __name__, # package name
            "examples/input1.cal"
        )
        mcla.build_from_cadlang(filename)

        st_prop = None
        inv_prop = None
        f_prop =  "C3 and not C4"
        query = MCLSimpleQuery(st_prop, inv_prop, f_prop)
        lsol = mcla.sq_solutions(query, 4, 1, mcla.unfolder.frontier_values)
        sol = lsol[0]
        assert len(lsol) == 2
        ic_seq = sol.extract_act_input_clock_seq()
        expected_sol = [[2], [], []]
        res = lit_sol_equal(expected_sol, ic_seq)
        print("expected ic_seq:", expected_sol)
        print("found ic_seq:", ic_seq)
        self.assert_(res, 'Error clock/input sequence for C3 and not C4')

        f_prop = "C4 and C3"
        query = MCLSimpleQuery(None, None, f_prop)
        lsol = mcla.sq_solutions(query, 4, 1, mcla.unfolder.frontier_values)
        sol = lsol[0]
        res = len(lsol) == 1
        ic_seq = sol.extract_act_input_clock_seq()
        #print ic_seq
        expected_sol = [[], [2], [], []]
        res = res and lit_sol_equal(expected_sol, ic_seq)
        self.assert_(res, 'Error clock/input sequence for C3 and C4')


    def test_unflatten(self):
        """
        Test unflatten trajectory
        """
        rep = ErrorRep()
        mcla = MCLAnalyser(rep)
        filename = pkg_resources.resource_filename(
            __name__, # package name
            "examples/delay1.cal"
        )
        mcla.build_from_cadlang(filename)

        st_prop = None
        inv_prop = None
        f_prop = "C3"
        query = MCLSimpleQuery(st_prop, inv_prop, f_prop)
        lsol = mcla.sq_solutions(query, 4, 1, mcla.unfolder.frontier_values)
        ufl = lsol[0].unflatten()
        expected_ufl = [[-1, -2, 3, -4, -5, -6, -7, -8, -9, -10, -11, -12, 13, 14, -15, 16], [-1, -2, -3, -4, -5, -6, -7, -8, 9, -10, -11, 12, -13, -14, -15, -16], [-1, -2, -3, -4, -5, -6, -7, -8, -9, 10, 11, 12, -13, -14, 15, -16]]
        res = lit_sol_equal(expected_ufl, ufl)
        self.assert_(res, 'Error unflatten')


    @unittest.skip("Test files not provided")
    def test_dimacs_frontier(self):
        """
        frontier solution in DIMACS form
        """
        rep = ErrorRep()
        mcla = MCLAnalyser(rep)
        prop = 'p15INK4b and p21CIP1'
        query = MCLSimpleQuery(None, None, prop)

        mcla.build_from_chart_file("../ucl/examples/tgf_211011_man.bcx")
        # Get <tuple <DimacsFrontierSol>>
        lsol = mcla.__sq_dimacs_frontier_solutions(query, 8, 1)
#        lforbidden = []
#        for sol in lsol:
#            forbid = []
#            for varcod in sol.frontier_values:
#                if varcod > 0:
#                    forbid.append(-varcod)
#            lforbidden.append(forbid)
        forbidden_sol = \
            [
                # Get all activated frontiers in the DimacsFrontierSol
                [-var for var in dimacs_front_sol.activated_frontier_values]
                 for dimacs_front_sol in lsol
            ]
#        print(lforbidden)
## TODO: assert lforbidden
        print("new forbid", forbidden_sol)
        raise Exception("ici")

        # try another solution
        query.dim_start = forbidden_sol
        # Get <tuple <DimacsFrontierSol>>
        lsol2 = mcla.__sq_dimacs_frontier_solutions(query, 8, 1)
        # comparison: solution are ordered and must be different
        # remains true if all components are equal (equal solutions)
        res = set(lsol) == set(lsol2)
        self.assert_(not res, 'Error dimacs_frontier: solutions not different')

    @unittest.skip("Test files not provided")
    def test_prune(self):
        """
        pruning a solution
        """
        rep = ErrorRep()
        mcla = MCLAnalyser(rep)
        prop = 'p15INK4b and p21CIP1'
        query = MCLSimpleQuery(None, None, prop)

        mcla.build_from_chart_file("../ucl/examples/tgf_211011_man.bcx")
        # lsol: list<dimacs frontier solution>
        lsol = mcla.__sq_dimacs_frontier_solutions(query, 8, 10)
        small_sol = mcla.less_activated_solution(lsol)
        cpt1 = 0
        for varcod in small_sol.get_solution():
            if varcod > 0:
                cpt1 += 1
        # check solution
        start = []
        for ssol in small_sol.frontier_values:
            start.append([ssol])
        query.dim_start = start
        res = mcla.sq_is_satisfiable(query, 8)
        self.assert_(res, 'Small sol is not a solution')
        # find if there are solutions with same unactivated places
        for ssol in small_sol.frontier_values:
            if ssol < 0:
                start.append([ssol])
        query.dim_start = start
        res = mcla.sq_is_satisfiable(query, 8)
        self.assert_(res, 'No solution with same unactivated places')
        pruned = mcla.__prune_frontier_solution(small_sol, query, 8)
        # compare sizes
        cpt2 = 0
        for varcod in prune.get_solution():
            if varcod > 0:
                cpt2 += 1
        res = (cpt2 <= cpt1)
        self.assert_(res, 'Prune did not reduce nb of activated places')




    def test_mac_no_clock(self):
        """
        test mac without clocks
        """
        # solving
        rep = ErrorRep()
        mcla = MCLAnalyser(rep)
        prop = 'p15INK4b and p21CIP1'
        filename = pkg_resources.resource_filename(
            __name__, # package name
            "../../guard_transitions/translators/tests/tgf_cano_noclock_cmp.cal"
        )
        mcla.build_from_cadlang(filename)
        mcla.unfolder.set_stats()
        query = MCLSimpleQuery(None, None, prop)
        mac_list = tuple(mcla.mac_search(query, 10))
        res = len(mac_list) == 8
        self.assert_(res, 'Incorrect number of mac')
        expected = [
            frozenset({'SP1', 'betaglycan__dimer___intToMb', 'SMAD4_nucl', 'importinbeta', 'MIZ_1', 'DAB2', 'TGFBfamily__dimer___active_exCellRegion', 'TGFBR2__dimer___active_intToMb', 'Axin_SMAD3', 'SMAD4_cy', 'SMAD2_3', 'FKBP12_TGFBR1__dimer___inactive_intToMb', 'CTGF'}),
            frozenset({'SP1', 'betaglycan__dimer___intToMb', 'SMAD4_nucl', 'MIZ_1', 'DAB2', 'TGFBfamily__dimer___active_exCellRegion', 'NUP214', 'TGFBR2__dimer___active_intToMb', 'Axin_SMAD3', 'SMAD4_cy', 'SMAD2_3', 'FKBP12_TGFBR1__dimer___inactive_intToMb', 'CTGF'}),
            frozenset({'SP1', 'betaglycan__dimer___intToMb', 'SMAD4_nucl', 'MIZ_1', 'DAB2', 'TGFBfamily__dimer___active_exCellRegion', 'TGFBR2__dimer___active_intToMb', 'Axin_SMAD3', 'SMAD4_cy', 'SMAD2_3', 'NUP153', 'FKBP12_TGFBR1__dimer___inactive_intToMb', 'CTGF'}),
            frozenset({'SP1', 'betaglycan__dimer___intToMb', 'SMAD4_nucl', 'MIZ_1', 'DAB2', 'TGFBfamily__dimer___active_exCellRegion', 'TGFBR2__dimer___active_intToMb', 'Axin_SMAD3', 'SMAD4_cy', 'importinalpha', 'SMAD2_3', 'FKBP12_TGFBR1__dimer___inactive_intToMb', 'CTGF'}),
            frozenset({'SP1', 'betaglycan__dimer___intToMb', 'SMAD4_nucl', 'FOXO1_3a_4_active_nucl', 'DAB2', 'CEBPB', 'TGFBfamily__dimer___active_exCellRegion', 'TGFBR2__dimer___active_intToMb', 'Axin_SMAD3', 'SMAD4_cy', 'SMAD2_3', 'NUP153', 'FKBP12_TGFBR1__dimer___inactive_intToMb', 'CTGF', 'FOXG1'}),
            frozenset({'SP1', 'betaglycan__dimer___intToMb', 'SMAD4_nucl', 'importinbeta', 'FOXO1_3a_4_active_nucl', 'DAB2', 'CEBPB', 'TGFBfamily__dimer___active_exCellRegion', 'TGFBR2__dimer___active_intToMb', 'Axin_SMAD3', 'SMAD4_cy', 'SMAD2_3', 'FKBP12_TGFBR1__dimer___inactive_intToMb', 'CTGF', 'FOXG1'}),
            frozenset({'SP1', 'betaglycan__dimer___intToMb', 'SMAD4_nucl', 'FOXO1_3a_4_active_nucl', 'DAB2', 'CEBPB', 'TGFBfamily__dimer___active_exCellRegion', 'NUP214', 'TGFBR2__dimer___active_intToMb', 'Axin_SMAD3', 'SMAD4_cy', 'SMAD2_3', 'FKBP12_TGFBR1__dimer___inactive_intToMb', 'CTGF', 'FOXG1'}),
            frozenset({'SP1', 'betaglycan__dimer___intToMb', 'SMAD4_nucl', 'FOXO1_3a_4_active_nucl', 'DAB2', 'CEBPB', 'TGFBfamily__dimer___active_exCellRegion', 'TGFBR2__dimer___active_intToMb', 'Axin_SMAD3', 'SMAD4_cy', 'importinalpha', 'SMAD2_3', 'FKBP12_TGFBR1__dimer___inactive_intToMb', 'CTGF', 'FOXG1'})
        ]
        found = [front_sol.activated_frontier for front_sol in mac_list]
        print("found test_mac_no_clock", found)
        # Forget the order with sets...
        res = set(expected) == set(found)
        self.assert_(res, 'Incorrect list of MAC (no clock)')


    def test_mac_clock(self):
        """
        mac with clocks
        """
        # solving
        rep = ErrorRep()
        mcla = MCLAnalyser(rep)
        prop = 'p15INK4b'
        #p = "C3"
        filename = pkg_resources.resource_filename(
            __name__, # package name
            "../../guard_transitions/translators/tests/tgf_cano_cmp.cal"
        )
        mcla.build_from_cadlang(filename)
        #mcla.build_from_cadlang("examples/delay2.cal")
        #mcla.unfolder.set_stats()
        query = MCLSimpleQuery(None, None, prop)
        # return iterable of FrontierSolution objects
        mac_list = tuple(mcla.mac_search(query, 10))
        expected = [
            frozenset({'SP1', 'SMAD4_nucl', 'MIZ_1', 'TGFB_TGFBR2_TGFBR1_DAB2_active_intToMb', 'SMAD2_3'}),
            frozenset({'SMAD4_nucl', 'FOXO1_3a_4_active_nucl', 'CEBPB', 'TGFB_TGFBR2_TGFBR1_DAB2_active_intToMb', 'SMAD2_3', 'FOXG1'}),
            frozenset({'SP1', 'betaglycan__dimer___intToMb', 'SMAD4_nucl', 'Caveolin_1', 'MIZ_1', 'TRAP_1_TGFBR1__dimer___inactive_intToMb', 'AP2B2', 'TGFBR2__dimer___active_intToMb', 'PML_SARA_SMAD2_3_earlyendosome', 'TGFBfamily__dimer___active_exCellRegion', 'SMAD7_SMURF1_2_nucl', 'CTGF'}),
            frozenset({'betaglycan__dimer___intToMb', 'SMAD4_nucl', 'Caveolin_1', 'FOXO1_3a_4_active_nucl', 'TRAP_1_TGFBR1__dimer___inactive_intToMb', 'AP2B2', 'CEBPB', 'TGFBR2__dimer___active_intToMb', 'PML_SARA_SMAD2_3_earlyendosome', 'TGFBfamily__dimer___active_exCellRegion', 'SMAD7_SMURF1_2_nucl', 'CTGF', 'FOXG1'})
        ]
        found = [front_sol.activated_frontier for front_sol in mac_list]
        print("found test_mac_no_clock", found)
        # Forget the order with sets...
        res = set(expected) == set(found)
        self.assert_(res, 'Incorrect list of MAC (clock)')


    def test_valid_simul_delay(self):
        """
        Test simulation of the delay example
        """
        rep = ErrorRep()
        mcla = MCLAnalyser(rep)
        filename = pkg_resources.resource_filename(
            __name__, # package name
            "examples/delay1.cal"
        )
        mcla.build_from_cadlang(filename)

        st_prop = None
        inv_prop = None
        f_prop =  "C3 and  C4"
        query = MCLSimpleQuery(st_prop, inv_prop, f_prop)
        # lsol: list<RawSolution>
        lsol = mcla.sq_solutions(query, 4, 1, mcla.unfolder.frontier_values)
        sol = lsol[0]
        f_sol = FrontierSolution.from_raw(sol)

        # independant simulation
        chm = ChartModel(" ")
        chm.build_from_cadlang(filename, rep)
        sim = ChartSimulator()
        sim.build(chm, True, rep)
        sim.simul_init_places(f_sol.activated_frontier)
        sim.set_act_input_stream(f_sol.ic_sequence)
        sim.set_reachable_prop(f_prop)

        while True:
            try:
                sim.simul_step()
            except SimulStopException as exc:
                exc_txt = exc.__str__()
                self.assert_(exc_txt == 'STOP', "Error: " + exc_txt)
                break
        # test if property reached
        res = sim.check_reachable_property()
        self.assert_(res, "Property not reached")


    def test_valid_simul_input(self):
        """
        Test simulation of the input example
        """
        rep = ErrorRep()
        mcla = MCLAnalyser(rep)
        filename = pkg_resources.resource_filename(
            __name__, # package name
            "examples/input1.cal"
        )
        mcla.build_from_cadlang(filename)

        st_prop = None
        inv_prop = None
        f_prop =  "C3 and  C4"
        query = MCLSimpleQuery(st_prop, inv_prop, f_prop)
        lsol = mcla.sq_solutions(query, 4, 1, mcla.unfolder.frontier_values)
        sol = lsol[0]
        f_sol = FrontierSolution.from_raw(sol)

        # independant simulation
        chm = ChartModel(" ")
        chm.build_from_cadlang(filename, rep)
        sim = ChartSimulator()
        sim.build(chm, True, rep)
        sim.simul_init_places(f_sol.activated_frontier)
        sim.set_act_input_stream(f_sol.ic_sequence)
        sim.set_reachable_prop(f_prop)

        while True:
            try:
                sim.simul_step()
            except SimulStopException as exc:
                exc_txt = exc.__str__()
                self.assert_(exc_txt == 'STOP', "Error: " + exc_txt)
                break
        # test if property reached
        res = sim.check_reachable_property()
        self.assert_(res, "Property not reached")


    def test_mac_delay_save(self):
        """Test save on the delay example

        x_file content:
            A1 B1 C1
                %
                % h2
                %
                %
        """
        rep = ErrorRep()
        mcla = MCLAnalyser(rep)
        filename = pkg_resources.resource_filename(
            __name__, # package name
            "examples/delay1.cal"
        )
        mcla.build_from_cadlang(filename)

        f_prop =  "C3 and C4"
        query = MCLSimpleQuery(None, None, f_prop)
        macwic = tuple(mcla.mac_search(query, 10))

        # Just save the first solution
        with open(DIR_LOGS + "x_file", 'w') as f_out:
    #        print macwic[0].activated_frontier
    #        print macwic[0].ic_sequence
    #        print '---------------'
            macwic[0].save(f_out)

        # Note: use cadbiom_cmd/tools/solutions.py functions to reload frontiers
        # Just reload the first solution
        r_front = FrontierSolution.from_file(DIR_LOGS + "x_file")
        expected = {'A1', 'B1', 'C1'}
        assert r_front.activated_frontier == expected, "Bad activated frontiers from file"
        expected = ['%', '% h2', '%', '%']
        assert r_front.ic_sequence == expected, "Bad ic_sequence from file"
        assert r_front.current_step == 4, "Bad max steps from file"

        res = macwic[0].activated_frontier == r_front.activated_frontier
        res = res and macwic[0].ic_sequence == r_front.ic_sequence
        res = res and r_front == macwic[0]
        self.assert_(res, "Frontier solutions differ after read")
        os.remove(DIR_LOGS + "x_file")


    def test_mcl_query_merge(self):
        """Test zone for some MCLSimpleQuery merge functions used in
        MCLAnalyser.is_strong_activator()
        """
        ## Setup
        rep = ErrorRep()
        mcla = MCLAnalyser(rep)
        filename = pkg_resources.resource_filename(
            __name__, # package name
            "examples/delay3.cal"
        )
        mcla.build_from_cadlang(filename)

        f_prop =  "C3 and C4"
        query = MCLSimpleQuery(None, None, f_prop)
        frontier_sols = tuple(mcla.mac_search(query, 10))

        ## Begin tests
        query_1 = MCLSimpleQuery.from_frontier_sol_same_timing(frontier_sols[0], mcla.unfolder)

        # Test interpretation of activated_frontier into human readable format
        expected = 'A1 and C1 and B1'
        assert query_1.start_prop == expected

        # Test interpretation of ic_sequence into variant property
        expected = ['', 'h2', '', '']
        assert query_1.variant_prop == expected

        # Test interpretation of activated_frontier into start property in DIMACS format
        expected = [[-2]]
        assert query_1.dim_start == expected

        assert query_1.final_prop == None
        assert query_1.inv_prop == None

        inv_prop = 'not(' + query.final_prop + ')'
        query_2 = MCLSimpleQuery(None, inv_prop, None)

        expected = "not(C3 and C4)"
        assert query_2.inv_prop == expected

        # Test merge of queries
        query_1 = query_2.merge(query_1)
        #(None, 'not(C3 and C4)', 'A1 and C1 and B1', ['', 'h2', '', '']
        expected = ['', 'h2', '', '']
        assert query_1.variant_prop == expected


        # Test merge of queries with both variant properties
        # 2 empty, 1 empty and not the other, none empty
        query_1.variant_prop = ['h2', 'h2', '', '']
        query_2.variant_prop = ['', 'h2', '', 'h2']
        query_1 = query_2.merge(query_1)
        expected = ['h2', 'h2 and h2', '', 'h2']
        assert query_1.variant_prop == expected


    def test_mac_inhibitor(self):
        """
        test
        [A1, B1, C1] not strong activator, D is a strong inhibitor
        """
        rep = ErrorRep()
        mcla = MCLAnalyser(rep)
        filename = pkg_resources.resource_filename(
            __name__, # package name
            "examples/delay3.cal"
        )
        mcla.build_from_cadlang(filename)

        f_prop =  "C3 and C4"
        query = MCLSimpleQuery(None, None, f_prop)
        frontier_sols = tuple(mcla.mac_search(query, 10))
        exp_act_front = {'A1', 'C1', 'B1'}
        obt_act_front = frontier_sols[0].activated_frontier
        res = exp_act_front == obt_act_front
        self.assert_(res, "Bad mac frontier solution")
        exp_timing = ["%", "% h2", "%", "%"]
        res = lit_subsol_equal(exp_timing, frontier_sols[0].ic_sequence)
        self.assert_(res, "Bad mac timing solution")


        # Strong activator search
        # return True
        res = mcla.is_strong_activator(frontier_sols[0], query)
        print(res)
        # it is not a strong activator
        # Refactor Note:
        # switch to res instead of not res; i think there is a logical inversion here
        self.assert_(res, "Not a strong activator")

        # Get FrontierSolutions
        inhib = mcla.mac_inhibitor_search(frontier_sols[0], query, 10)
        print('INHIB: found', len(inhib))
        for isol in inhib:
            print("Inhib sol:", isol)

        # Solution 1: {'D', 'A1', 'C1', 'B1'} or {'A1', 'C1', 'B1'}
        exp_act_front1 = {'D', 'A1', 'C1', 'B1'}
        obt_act_front = inhib[0].activated_frontier
        obt_timing = inhib[0].ic_sequence
        res = exp_act_front1 == obt_act_front
        # 2 possible timings
        if res:
            # not used
            exp_timing = ["%", "% h2", "%", "%"]
            res = res and lit_subsol_equal(exp_timing, obt_timing)
            self.assert_(res, "Bad inhib timing solution")
        else:
            # Condition used (sol: {'A1', 'C1', 'B1'})
            res = exp_act_front == obt_act_front
            exp_timing = ["% h2", "% h2", "%", "%"]
            res = res and lit_subsol_equal(exp_timing, obt_timing)
            self.assert_(res, "Bad inhib timing solution")

        # Solution 2: {'D', 'A1', 'C1', 'B1'} or {'A1', 'C1', 'B1'} (different of sol1)
#        obt_act_front = inhib[1].activated_frontier
#        obt_timing = inhib[1].ic_sequence
#        res = exp_act_front == obt_act_front
#        # 2 possible timings
#        if res:
#            exp_timing = ["% h2", "% h2", "%", "%"]
#            res = res and lit_subsol_equal(exp_timing, obt_timing)
#            self.assert_(res, "Bad inhib timing solution")
#        else:
#            res = exp_act_front1 == obt_act_front
#            exp_timing = ["%", "% h2", "%", "%"]
#            res = res and lit_subsol_equal(exp_timing, obt_timing)
#            self.assert_(res, "Bad inhib timing solution")

        # Strong inhibitor search
        # return False
        res = mcla.is_strong_inhibitor(inhib[0], query)

        query_1 = MCLSimpleQuery.from_frontier_sol_new_timing(inhib[0], mcla.unfolder)
        found = query_1.start_prop
        assert "A1 and C1 and B1" == found
        found = query_1.inv_prop
        assert found == None
        found == query_1.final_prop
        assert found == None
        found = query_1.variant_prop
        assert found == ['not (h2)', 'not (h2)', '', '']
        # Inactivation of of other frontier places (not used in the solution)
        found = query_1.dim_start
        assert found == [[-2]] # D

        print("is strong inhibitor ?", res)
        # it is a strong inhibitor
        # Refactor Note:
        # Switch to not res instead of res; i think there is a logical inversion here
        self.assert_(not res, "Bad answer - strong inhibitor")





# $ANTLR 3.5.2 cadbiom/models/guard_transitions/condexp.g 2018-12-14 16:41:06

import sys
from antlr3 import *
from antlr3.compat import set, frozenset



# for convenience in actions
HIDDEN = BaseRecognizer.HIDDEN

# token types
EOF=-1
AND=4
COMMENT=5
DIGIT=6
DOL=7
F=8
IDENT=9
LETTER=10
NOT=11
OR=12
PD=13
PG=14
T=15
WS=16


class condexpLexer(Lexer):

    grammarFileName = "cadbiom/models/guard_transitions/condexp.g"
    api_version = 1

    def __init__(self, input=None, state=None):
        if state is None:
            state = RecognizerSharedState()
        super(condexpLexer, self).__init__(input, state)

        self.delegates = []






    # $ANTLR start "WS"
    def mWS(self, ):
        try:
            _type = WS
            _channel = DEFAULT_CHANNEL

            # cadbiom/models/guard_transitions/condexp.g:55:13: ( ( ' ' | '\\t' | '\\n' ) )
            # cadbiom/models/guard_transitions/condexp.g:55:16: ( ' ' | '\\t' | '\\n' )
            pass 
            if (9 <= self.input.LA(1) <= 10) or self.input.LA(1) == 32:
                self.input.consume()
            else:
                mse = MismatchedSetException(None, self.input)
                self.recover(mse)
                raise mse



            #action start
            _channel = HIDDEN;
            #action end




            self._state.type = _type
            self._state.channel = _channel
        finally:
            pass

    # $ANTLR end "WS"



    # $ANTLR start "COMMENT"
    def mCOMMENT(self, ):
        try:
            _type = COMMENT
            _channel = DEFAULT_CHANNEL

            # cadbiom/models/guard_transitions/condexp.g:57:13: ( '//' (~ '\\n' )* '\\n' )
            # cadbiom/models/guard_transitions/condexp.g:57:15: '//' (~ '\\n' )* '\\n'
            pass 
            self.match("//")


            # cadbiom/models/guard_transitions/condexp.g:57:19: (~ '\\n' )*
            while True: #loop1
                alt1 = 2
                LA1_0 = self.input.LA(1)

                if ((0 <= LA1_0 <= 9) or (11 <= LA1_0 <= 65535)) :
                    alt1 = 1


                if alt1 == 1:
                    # cadbiom/models/guard_transitions/condexp.g:
                    pass 
                    if (0 <= self.input.LA(1) <= 9) or (11 <= self.input.LA(1) <= 65535):
                        self.input.consume()
                    else:
                        mse = MismatchedSetException(None, self.input)
                        self.recover(mse)
                        raise mse




                else:
                    break #loop1


            self.match(10)

            #action start
            _channel = HIDDEN;
            #action end




            self._state.type = _type
            self._state.channel = _channel
        finally:
            pass

    # $ANTLR end "COMMENT"



    # $ANTLR start "AND"
    def mAND(self, ):
        try:
            _type = AND
            _channel = DEFAULT_CHANNEL

            # cadbiom/models/guard_transitions/condexp.g:60:9: ( 'and' )
            # cadbiom/models/guard_transitions/condexp.g:60:11: 'and'
            pass 
            self.match("and")




            self._state.type = _type
            self._state.channel = _channel
        finally:
            pass

    # $ANTLR end "AND"



    # $ANTLR start "OR"
    def mOR(self, ):
        try:
            _type = OR
            _channel = DEFAULT_CHANNEL

            # cadbiom/models/guard_transitions/condexp.g:61:9: ( 'or' )
            # cadbiom/models/guard_transitions/condexp.g:61:11: 'or'
            pass 
            self.match("or")




            self._state.type = _type
            self._state.channel = _channel
        finally:
            pass

    # $ANTLR end "OR"



    # $ANTLR start "NOT"
    def mNOT(self, ):
        try:
            _type = NOT
            _channel = DEFAULT_CHANNEL

            # cadbiom/models/guard_transitions/condexp.g:62:9: ( 'not' )
            # cadbiom/models/guard_transitions/condexp.g:62:11: 'not'
            pass 
            self.match("not")




            self._state.type = _type
            self._state.channel = _channel
        finally:
            pass

    # $ANTLR end "NOT"



    # $ANTLR start "T"
    def mT(self, ):
        try:
            _type = T
            _channel = DEFAULT_CHANNEL

            # cadbiom/models/guard_transitions/condexp.g:63:9: ( 'true' )
            # cadbiom/models/guard_transitions/condexp.g:63:11: 'true'
            pass 
            self.match("true")




            self._state.type = _type
            self._state.channel = _channel
        finally:
            pass

    # $ANTLR end "T"



    # $ANTLR start "F"
    def mF(self, ):
        try:
            _type = F
            _channel = DEFAULT_CHANNEL

            # cadbiom/models/guard_transitions/condexp.g:64:9: ( 'false' )
            # cadbiom/models/guard_transitions/condexp.g:64:11: 'false'
            pass 
            self.match("false")




            self._state.type = _type
            self._state.channel = _channel
        finally:
            pass

    # $ANTLR end "F"



    # $ANTLR start "PG"
    def mPG(self, ):
        try:
            _type = PG
            _channel = DEFAULT_CHANNEL

            # cadbiom/models/guard_transitions/condexp.g:66:9: ( '(' )
            # cadbiom/models/guard_transitions/condexp.g:66:11: '('
            pass 
            self.match(40)



            self._state.type = _type
            self._state.channel = _channel
        finally:
            pass

    # $ANTLR end "PG"



    # $ANTLR start "PD"
    def mPD(self, ):
        try:
            _type = PD
            _channel = DEFAULT_CHANNEL

            # cadbiom/models/guard_transitions/condexp.g:67:9: ( ')' )
            # cadbiom/models/guard_transitions/condexp.g:67:11: ')'
            pass 
            self.match(41)



            self._state.type = _type
            self._state.channel = _channel
        finally:
            pass

    # $ANTLR end "PD"



    # $ANTLR start "DOL"
    def mDOL(self, ):
        try:
            _type = DOL
            _channel = DEFAULT_CHANNEL

            # cadbiom/models/guard_transitions/condexp.g:68:9: ( '$' )
            # cadbiom/models/guard_transitions/condexp.g:68:11: '$'
            pass 
            self.match(36)



            self._state.type = _type
            self._state.channel = _channel
        finally:
            pass

    # $ANTLR end "DOL"



    # $ANTLR start "LETTER"
    def mLETTER(self, ):
        try:
            # cadbiom/models/guard_transitions/condexp.g:71:19: ( 'a' .. 'z' | 'A' .. 'Z' | '_' )
            # cadbiom/models/guard_transitions/condexp.g:
            pass 
            if (65 <= self.input.LA(1) <= 90) or self.input.LA(1) == 95 or (97 <= self.input.LA(1) <= 122):
                self.input.consume()
            else:
                mse = MismatchedSetException(None, self.input)
                self.recover(mse)
                raise mse






        finally:
            pass

    # $ANTLR end "LETTER"



    # $ANTLR start "DIGIT"
    def mDIGIT(self, ):
        try:
            # cadbiom/models/guard_transitions/condexp.g:73:19: ( '0' .. '9' )
            # cadbiom/models/guard_transitions/condexp.g:
            pass 
            if (48 <= self.input.LA(1) <= 57):
                self.input.consume()
            else:
                mse = MismatchedSetException(None, self.input)
                self.recover(mse)
                raise mse






        finally:
            pass

    # $ANTLR end "DIGIT"



    # $ANTLR start "IDENT"
    def mIDENT(self, ):
        try:
            _type = IDENT
            _channel = DEFAULT_CHANNEL

            # cadbiom/models/guard_transitions/condexp.g:75:9: ( ( LETTER | DIGIT )+ )
            # cadbiom/models/guard_transitions/condexp.g:75:11: ( LETTER | DIGIT )+
            pass 
            # cadbiom/models/guard_transitions/condexp.g:75:11: ( LETTER | DIGIT )+
            cnt2 = 0
            while True: #loop2
                alt2 = 2
                LA2_0 = self.input.LA(1)

                if ((48 <= LA2_0 <= 57) or (65 <= LA2_0 <= 90) or LA2_0 == 95 or (97 <= LA2_0 <= 122)) :
                    alt2 = 1


                if alt2 == 1:
                    # cadbiom/models/guard_transitions/condexp.g:
                    pass 
                    if (48 <= self.input.LA(1) <= 57) or (65 <= self.input.LA(1) <= 90) or self.input.LA(1) == 95 or (97 <= self.input.LA(1) <= 122):
                        self.input.consume()
                    else:
                        mse = MismatchedSetException(None, self.input)
                        self.recover(mse)
                        raise mse




                else:
                    if cnt2 >= 1:
                        break #loop2

                    eee = EarlyExitException(2, self.input)
                    raise eee

                cnt2 += 1




            self._state.type = _type
            self._state.channel = _channel
        finally:
            pass

    # $ANTLR end "IDENT"



    def mTokens(self):
        # cadbiom/models/guard_transitions/condexp.g:1:8: ( WS | COMMENT | AND | OR | NOT | T | F | PG | PD | DOL | IDENT )
        alt3 = 11
        LA3 = self.input.LA(1)
        if LA3 == 9 or LA3 == 10 or LA3 == 32:
            alt3 = 1
        elif LA3 == 47:
            alt3 = 2
        elif LA3 == 97:
            LA3_3 = self.input.LA(2)

            if (LA3_3 == 110) :
                LA3_12 = self.input.LA(3)

                if (LA3_12 == 100) :
                    LA3_17 = self.input.LA(4)

                    if ((48 <= LA3_17 <= 57) or (65 <= LA3_17 <= 90) or LA3_17 == 95 or (97 <= LA3_17 <= 122)) :
                        alt3 = 11
                    else:
                        alt3 = 3

                else:
                    alt3 = 11

            else:
                alt3 = 11

        elif LA3 == 111:
            LA3_4 = self.input.LA(2)

            if (LA3_4 == 114) :
                LA3_13 = self.input.LA(3)

                if ((48 <= LA3_13 <= 57) or (65 <= LA3_13 <= 90) or LA3_13 == 95 or (97 <= LA3_13 <= 122)) :
                    alt3 = 11
                else:
                    alt3 = 4

            else:
                alt3 = 11

        elif LA3 == 110:
            LA3_5 = self.input.LA(2)

            if (LA3_5 == 111) :
                LA3_14 = self.input.LA(3)

                if (LA3_14 == 116) :
                    LA3_19 = self.input.LA(4)

                    if ((48 <= LA3_19 <= 57) or (65 <= LA3_19 <= 90) or LA3_19 == 95 or (97 <= LA3_19 <= 122)) :
                        alt3 = 11
                    else:
                        alt3 = 5

                else:
                    alt3 = 11

            else:
                alt3 = 11

        elif LA3 == 116:
            LA3_6 = self.input.LA(2)

            if (LA3_6 == 114) :
                LA3_15 = self.input.LA(3)

                if (LA3_15 == 117) :
                    LA3_20 = self.input.LA(4)

                    if (LA3_20 == 101) :
                        LA3_24 = self.input.LA(5)

                        if ((48 <= LA3_24 <= 57) or (65 <= LA3_24 <= 90) or LA3_24 == 95 or (97 <= LA3_24 <= 122)) :
                            alt3 = 11
                        else:
                            alt3 = 6

                    else:
                        alt3 = 11

                else:
                    alt3 = 11

            else:
                alt3 = 11

        elif LA3 == 102:
            LA3_7 = self.input.LA(2)

            if (LA3_7 == 97) :
                LA3_16 = self.input.LA(3)

                if (LA3_16 == 108) :
                    LA3_21 = self.input.LA(4)

                    if (LA3_21 == 115) :
                        LA3_25 = self.input.LA(5)

                        if (LA3_25 == 101) :
                            LA3_27 = self.input.LA(6)

                            if ((48 <= LA3_27 <= 57) or (65 <= LA3_27 <= 90) or LA3_27 == 95 or (97 <= LA3_27 <= 122)) :
                                alt3 = 11
                            else:
                                alt3 = 7

                        else:
                            alt3 = 11

                    else:
                        alt3 = 11

                else:
                    alt3 = 11

            else:
                alt3 = 11

        elif LA3 == 40:
            alt3 = 8
        elif LA3 == 41:
            alt3 = 9
        elif LA3 == 36:
            alt3 = 10
        elif LA3 == 48 or LA3 == 49 or LA3 == 50 or LA3 == 51 or LA3 == 52 or LA3 == 53 or LA3 == 54 or LA3 == 55 or LA3 == 56 or LA3 == 57 or LA3 == 65 or LA3 == 66 or LA3 == 67 or LA3 == 68 or LA3 == 69 or LA3 == 70 or LA3 == 71 or LA3 == 72 or LA3 == 73 or LA3 == 74 or LA3 == 75 or LA3 == 76 or LA3 == 77 or LA3 == 78 or LA3 == 79 or LA3 == 80 or LA3 == 81 or LA3 == 82 or LA3 == 83 or LA3 == 84 or LA3 == 85 or LA3 == 86 or LA3 == 87 or LA3 == 88 or LA3 == 89 or LA3 == 90 or LA3 == 95 or LA3 == 98 or LA3 == 99 or LA3 == 100 or LA3 == 101 or LA3 == 103 or LA3 == 104 or LA3 == 105 or LA3 == 106 or LA3 == 107 or LA3 == 108 or LA3 == 109 or LA3 == 112 or LA3 == 113 or LA3 == 114 or LA3 == 115 or LA3 == 117 or LA3 == 118 or LA3 == 119 or LA3 == 120 or LA3 == 121 or LA3 == 122:
            alt3 = 11
        else:
            nvae = NoViableAltException("", 3, 0, self.input)

            raise nvae


        if alt3 == 1:
            # cadbiom/models/guard_transitions/condexp.g:1:10: WS
            pass 
            self.mWS()



        elif alt3 == 2:
            # cadbiom/models/guard_transitions/condexp.g:1:13: COMMENT
            pass 
            self.mCOMMENT()



        elif alt3 == 3:
            # cadbiom/models/guard_transitions/condexp.g:1:21: AND
            pass 
            self.mAND()



        elif alt3 == 4:
            # cadbiom/models/guard_transitions/condexp.g:1:25: OR
            pass 
            self.mOR()



        elif alt3 == 5:
            # cadbiom/models/guard_transitions/condexp.g:1:28: NOT
            pass 
            self.mNOT()



        elif alt3 == 6:
            # cadbiom/models/guard_transitions/condexp.g:1:32: T
            pass 
            self.mT()



        elif alt3 == 7:
            # cadbiom/models/guard_transitions/condexp.g:1:34: F
            pass 
            self.mF()



        elif alt3 == 8:
            # cadbiom/models/guard_transitions/condexp.g:1:36: PG
            pass 
            self.mPG()



        elif alt3 == 9:
            # cadbiom/models/guard_transitions/condexp.g:1:39: PD
            pass 
            self.mPD()



        elif alt3 == 10:
            # cadbiom/models/guard_transitions/condexp.g:1:42: DOL
            pass 
            self.mDOL()



        elif alt3 == 11:
            # cadbiom/models/guard_transitions/condexp.g:1:46: IDENT
            pass 
            self.mIDENT()








 



def main(argv, stdin=sys.stdin, stdout=sys.stdout, stderr=sys.stderr):
    from antlr3.main import LexerMain
    main = LexerMain(condexpLexer)

    main.stdin = stdin
    main.stdout = stdout
    main.stderr = stderr
    main.execute(argv)



if __name__ == '__main__':
    main(sys.argv)



# -*- coding: utf-8 -*-
"""Unit tests for CLDynSys"""

from __future__ import unicode_literals
from __future__ import print_function
import pytest

from cadbiom.models.clause_constraints.CLDynSys import Literal, Clause

@pytest.fixture()
def feed_literals():

    l1 = Literal("A", True)
    l2 = Literal("A", False)
    l3 = Literal("B", False)
    l4 = Literal("B", False)

    return l1, l2, l3, l4

@pytest.fixture()
def feed_clauses(feed_literals):

    l1, l2, l3, l4 = feed_literals

    clause1 = Clause([l1, l2, l3])
    clause2 = Clause([l1, l3, l2])
    clause3 = Clause([l1, l2])

    return clause1, clause2, clause3


def test_literals(feed_literals):

    l1, l2, l3, l4 = feed_literals

    # Assignment and representation
    expected = "A"
    assert str(l1) == expected

    expected = "not A"
    assert str(l2) == expected

    expected = "_A"
    assert l2.code_name() == expected

    # Equality
    assert l1 != l2
    found = l1 == l2
    assert found == False

    assert l3 == l4
    found = (l3 != l4)
    print((l3 != l4), (l3 == l4))
    assert found == False

    expected = True
    assert l1.opposite(l2) == expected


def test_clauses(feed_literals, feed_clauses):

    l1, l2, l3, l4 = feed_literals

    clause1, clause2, clause3 = feed_clauses

    # Equality
    assert clause1 == clause2
    found = (clause1 != clause2)
    assert found == False

    # Length
    assert clause1 != clause3
    assert clause1 > clause3
    #assert clause1 >= clause3
    found = clause1 < clause3
    assert found == False




# $ANTLR 3.5.2 cadbiom/models/guard_transitions/translators/pintlang.g 2018-12-14 16:41:07

import sys
from antlr3 import *
from antlr3.compat import set, frozenset



# for convenience in actions
HIDDEN = BaseRecognizer.HIDDEN

# token types
EOF=-1
AND=4
ARROW=5
COM=6
COMMENT=7
COOP=8
DIGIT=9
IDENT=10
INT=11
IN_KW=12
LB=13
LETTER=14
LP=15
NOT=16
OR=17
PROC=18
RB=19
RP=20
SC=21
WS=22


class pintlangLexer(Lexer):

    grammarFileName = "cadbiom/models/guard_transitions/translators/pintlang.g"
    api_version = 1

    def __init__(self, input=None, state=None):
        if state is None:
            state = RecognizerSharedState()
        super(pintlangLexer, self).__init__(input, state)

        self.delegates = []




                            
    def set_error_reporter(self, err):
        self.error_reporter = err

    def displayRecognitionError(self, tokenNames, re):
        hdr = self.getErrorHeader(re)
        msg = self.getErrorMessage(re, tokenNames)
        self.error_reporter.display(hdr,msg)
        

    def displayExceptionMessage(self, e):
        msg = self.getErrorMessage(self, e, tokenNames)
        self.error_reporter.display('',msg)



    # $ANTLR start "WS"
    def mWS(self, ):
        try:
            _type = WS
            _channel = DEFAULT_CHANNEL

            # cadbiom/models/guard_transitions/translators/pintlang.g:251:13: ( ( ' ' | '\\t' | '\\n' ) )
            # cadbiom/models/guard_transitions/translators/pintlang.g:251:16: ( ' ' | '\\t' | '\\n' )
            pass 
            if (9 <= self.input.LA(1) <= 10) or self.input.LA(1) == 32:
                self.input.consume()
            else:
                mse = MismatchedSetException(None, self.input)
                self.recover(mse)
                raise mse



            #action start
            _channel = HIDDEN;
            #action end




            self._state.type = _type
            self._state.channel = _channel
        finally:
            pass

    # $ANTLR end "WS"



    # $ANTLR start "COMMENT"
    def mCOMMENT(self, ):
        try:
            _type = COMMENT
            _channel = DEFAULT_CHANNEL

            # cadbiom/models/guard_transitions/translators/pintlang.g:253:13: ( '//' (~ '\\n' )* '\\n' )
            # cadbiom/models/guard_transitions/translators/pintlang.g:253:15: '//' (~ '\\n' )* '\\n'
            pass 
            self.match("//")


            # cadbiom/models/guard_transitions/translators/pintlang.g:253:19: (~ '\\n' )*
            while True: #loop1
                alt1 = 2
                LA1_0 = self.input.LA(1)

                if ((0 <= LA1_0 <= 9) or (11 <= LA1_0 <= 65535)) :
                    alt1 = 1


                if alt1 == 1:
                    # cadbiom/models/guard_transitions/translators/pintlang.g:
                    pass 
                    if (0 <= self.input.LA(1) <= 9) or (11 <= self.input.LA(1) <= 65535):
                        self.input.consume()
                    else:
                        mse = MismatchedSetException(None, self.input)
                        self.recover(mse)
                        raise mse




                else:
                    break #loop1


            self.match(10)

            #action start
            _channel = HIDDEN;
            #action end




            self._state.type = _type
            self._state.channel = _channel
        finally:
            pass

    # $ANTLR end "COMMENT"



    # $ANTLR start "PROC"
    def mPROC(self, ):
        try:
            _type = PROC
            _channel = DEFAULT_CHANNEL

            # cadbiom/models/guard_transitions/translators/pintlang.g:256:9: ( 'process' )
            # cadbiom/models/guard_transitions/translators/pintlang.g:256:11: 'process'
            pass 
            self.match("process")




            self._state.type = _type
            self._state.channel = _channel
        finally:
            pass

    # $ANTLR end "PROC"



    # $ANTLR start "ARROW"
    def mARROW(self, ):
        try:
            _type = ARROW
            _channel = DEFAULT_CHANNEL

            # cadbiom/models/guard_transitions/translators/pintlang.g:257:9: ( '->' )
            # cadbiom/models/guard_transitions/translators/pintlang.g:257:11: '->'
            pass 
            self.match("->")




            self._state.type = _type
            self._state.channel = _channel
        finally:
            pass

    # $ANTLR end "ARROW"



    # $ANTLR start "COOP"
    def mCOOP(self, ):
        try:
            _type = COOP
            _channel = DEFAULT_CHANNEL

            # cadbiom/models/guard_transitions/translators/pintlang.g:258:9: ( 'COOPERATIVITY' )
            # cadbiom/models/guard_transitions/translators/pintlang.g:258:11: 'COOPERATIVITY'
            pass 
            self.match("COOPERATIVITY")




            self._state.type = _type
            self._state.channel = _channel
        finally:
            pass

    # $ANTLR end "COOP"



    # $ANTLR start "IN_KW"
    def mIN_KW(self, ):
        try:
            _type = IN_KW
            _channel = DEFAULT_CHANNEL

            # cadbiom/models/guard_transitions/translators/pintlang.g:259:9: ( 'in' )
            # cadbiom/models/guard_transitions/translators/pintlang.g:259:11: 'in'
            pass 
            self.match("in")




            self._state.type = _type
            self._state.channel = _channel
        finally:
            pass

    # $ANTLR end "IN_KW"



    # $ANTLR start "LP"
    def mLP(self, ):
        try:
            _type = LP
            _channel = DEFAULT_CHANNEL

            # cadbiom/models/guard_transitions/translators/pintlang.g:261:9: ( '(' )
            # cadbiom/models/guard_transitions/translators/pintlang.g:261:11: '('
            pass 
            self.match(40)



            self._state.type = _type
            self._state.channel = _channel
        finally:
            pass

    # $ANTLR end "LP"



    # $ANTLR start "RP"
    def mRP(self, ):
        try:
            _type = RP
            _channel = DEFAULT_CHANNEL

            # cadbiom/models/guard_transitions/translators/pintlang.g:262:9: ( ')' )
            # cadbiom/models/guard_transitions/translators/pintlang.g:262:11: ')'
            pass 
            self.match(41)



            self._state.type = _type
            self._state.channel = _channel
        finally:
            pass

    # $ANTLR end "RP"



    # $ANTLR start "LB"
    def mLB(self, ):
        try:
            _type = LB
            _channel = DEFAULT_CHANNEL

            # cadbiom/models/guard_transitions/translators/pintlang.g:263:9: ( '[' )
            # cadbiom/models/guard_transitions/translators/pintlang.g:263:11: '['
            pass 
            self.match(91)



            self._state.type = _type
            self._state.channel = _channel
        finally:
            pass

    # $ANTLR end "LB"



    # $ANTLR start "RB"
    def mRB(self, ):
        try:
            _type = RB
            _channel = DEFAULT_CHANNEL

            # cadbiom/models/guard_transitions/translators/pintlang.g:264:9: ( ']' )
            # cadbiom/models/guard_transitions/translators/pintlang.g:264:11: ']'
            pass 
            self.match(93)



            self._state.type = _type
            self._state.channel = _channel
        finally:
            pass

    # $ANTLR end "RB"



    # $ANTLR start "SC"
    def mSC(self, ):
        try:
            _type = SC
            _channel = DEFAULT_CHANNEL

            # cadbiom/models/guard_transitions/translators/pintlang.g:265:9: ( ';' )
            # cadbiom/models/guard_transitions/translators/pintlang.g:265:11: ';'
            pass 
            self.match(59)



            self._state.type = _type
            self._state.channel = _channel
        finally:
            pass

    # $ANTLR end "SC"



    # $ANTLR start "COM"
    def mCOM(self, ):
        try:
            _type = COM
            _channel = DEFAULT_CHANNEL

            # cadbiom/models/guard_transitions/translators/pintlang.g:266:9: ( ',' )
            # cadbiom/models/guard_transitions/translators/pintlang.g:266:11: ','
            pass 
            self.match(44)



            self._state.type = _type
            self._state.channel = _channel
        finally:
            pass

    # $ANTLR end "COM"



    # $ANTLR start "AND"
    def mAND(self, ):
        try:
            _type = AND
            _channel = DEFAULT_CHANNEL

            # cadbiom/models/guard_transitions/translators/pintlang.g:268:9: ( 'and' )
            # cadbiom/models/guard_transitions/translators/pintlang.g:268:11: 'and'
            pass 
            self.match("and")




            self._state.type = _type
            self._state.channel = _channel
        finally:
            pass

    # $ANTLR end "AND"



    # $ANTLR start "OR"
    def mOR(self, ):
        try:
            _type = OR
            _channel = DEFAULT_CHANNEL

            # cadbiom/models/guard_transitions/translators/pintlang.g:269:9: ( 'or' )
            # cadbiom/models/guard_transitions/translators/pintlang.g:269:11: 'or'
            pass 
            self.match("or")




            self._state.type = _type
            self._state.channel = _channel
        finally:
            pass

    # $ANTLR end "OR"



    # $ANTLR start "NOT"
    def mNOT(self, ):
        try:
            _type = NOT
            _channel = DEFAULT_CHANNEL

            # cadbiom/models/guard_transitions/translators/pintlang.g:270:9: ( 'not' )
            # cadbiom/models/guard_transitions/translators/pintlang.g:270:11: 'not'
            pass 
            self.match("not")




            self._state.type = _type
            self._state.channel = _channel
        finally:
            pass

    # $ANTLR end "NOT"



    # $ANTLR start "LETTER"
    def mLETTER(self, ):
        try:
            # cadbiom/models/guard_transitions/translators/pintlang.g:272:19: ( 'a' .. 'z' | 'A' .. 'Z' | '_' )
            # cadbiom/models/guard_transitions/translators/pintlang.g:
            pass 
            if (65 <= self.input.LA(1) <= 90) or self.input.LA(1) == 95 or (97 <= self.input.LA(1) <= 122):
                self.input.consume()
            else:
                mse = MismatchedSetException(None, self.input)
                self.recover(mse)
                raise mse






        finally:
            pass

    # $ANTLR end "LETTER"



    # $ANTLR start "DIGIT"
    def mDIGIT(self, ):
        try:
            # cadbiom/models/guard_transitions/translators/pintlang.g:274:19: ( '0' .. '9' )
            # cadbiom/models/guard_transitions/translators/pintlang.g:
            pass 
            if (48 <= self.input.LA(1) <= 57):
                self.input.consume()
            else:
                mse = MismatchedSetException(None, self.input)
                self.recover(mse)
                raise mse






        finally:
            pass

    # $ANTLR end "DIGIT"



    # $ANTLR start "IDENT"
    def mIDENT(self, ):
        try:
            _type = IDENT
            _channel = DEFAULT_CHANNEL

            # cadbiom/models/guard_transitions/translators/pintlang.g:276:9: ( LETTER ( LETTER | DIGIT )* )
            # cadbiom/models/guard_transitions/translators/pintlang.g:276:11: LETTER ( LETTER | DIGIT )*
            pass 
            self.mLETTER()


            # cadbiom/models/guard_transitions/translators/pintlang.g:276:17: ( LETTER | DIGIT )*
            while True: #loop2
                alt2 = 2
                LA2_0 = self.input.LA(1)

                if ((48 <= LA2_0 <= 57) or (65 <= LA2_0 <= 90) or LA2_0 == 95 or (97 <= LA2_0 <= 122)) :
                    alt2 = 1


                if alt2 == 1:
                    # cadbiom/models/guard_transitions/translators/pintlang.g:
                    pass 
                    if (48 <= self.input.LA(1) <= 57) or (65 <= self.input.LA(1) <= 90) or self.input.LA(1) == 95 or (97 <= self.input.LA(1) <= 122):
                        self.input.consume()
                    else:
                        mse = MismatchedSetException(None, self.input)
                        self.recover(mse)
                        raise mse




                else:
                    break #loop2




            self._state.type = _type
            self._state.channel = _channel
        finally:
            pass

    # $ANTLR end "IDENT"



    # $ANTLR start "INT"
    def mINT(self, ):
        try:
            _type = INT
            _channel = DEFAULT_CHANNEL

            # cadbiom/models/guard_transitions/translators/pintlang.g:278:9: ( ( DIGIT )+ )
            # cadbiom/models/guard_transitions/translators/pintlang.g:278:11: ( DIGIT )+
            pass 
            # cadbiom/models/guard_transitions/translators/pintlang.g:278:11: ( DIGIT )+
            cnt3 = 0
            while True: #loop3
                alt3 = 2
                LA3_0 = self.input.LA(1)

                if ((48 <= LA3_0 <= 57)) :
                    alt3 = 1


                if alt3 == 1:
                    # cadbiom/models/guard_transitions/translators/pintlang.g:
                    pass 
                    if (48 <= self.input.LA(1) <= 57):
                        self.input.consume()
                    else:
                        mse = MismatchedSetException(None, self.input)
                        self.recover(mse)
                        raise mse




                else:
                    if cnt3 >= 1:
                        break #loop3

                    eee = EarlyExitException(3, self.input)
                    raise eee

                cnt3 += 1




            self._state.type = _type
            self._state.channel = _channel
        finally:
            pass

    # $ANTLR end "INT"



    def mTokens(self):
        # cadbiom/models/guard_transitions/translators/pintlang.g:1:8: ( WS | COMMENT | PROC | ARROW | COOP | IN_KW | LP | RP | LB | RB | SC | COM | AND | OR | NOT | IDENT | INT )
        alt4 = 17
        LA4 = self.input.LA(1)
        if LA4 == 9 or LA4 == 10 or LA4 == 32:
            alt4 = 1
        elif LA4 == 47:
            alt4 = 2
        elif LA4 == 112:
            LA4_3 = self.input.LA(2)

            if (LA4_3 == 114) :
                LA4_18 = self.input.LA(3)

                if (LA4_18 == 111) :
                    LA4_24 = self.input.LA(4)

                    if (LA4_24 == 99) :
                        LA4_30 = self.input.LA(5)

                        if (LA4_30 == 101) :
                            LA4_34 = self.input.LA(6)

                            if (LA4_34 == 115) :
                                LA4_36 = self.input.LA(7)

                                if (LA4_36 == 115) :
                                    LA4_38 = self.input.LA(8)

                                    if ((48 <= LA4_38 <= 57) or (65 <= LA4_38 <= 90) or LA4_38 == 95 or (97 <= LA4_38 <= 122)) :
                                        alt4 = 16
                                    else:
                                        alt4 = 3

                                else:
                                    alt4 = 16

                            else:
                                alt4 = 16

                        else:
                            alt4 = 16

                    else:
                        alt4 = 16

                else:
                    alt4 = 16

            else:
                alt4 = 16

        elif LA4 == 45:
            alt4 = 4
        elif LA4 == 67:
            LA4_5 = self.input.LA(2)

            if (LA4_5 == 79) :
                LA4_19 = self.input.LA(3)

                if (LA4_19 == 79) :
                    LA4_25 = self.input.LA(4)

                    if (LA4_25 == 80) :
                        LA4_31 = self.input.LA(5)

                        if (LA4_31 == 69) :
                            LA4_35 = self.input.LA(6)

                            if (LA4_35 == 82) :
                                LA4_37 = self.input.LA(7)

                                if (LA4_37 == 65) :
                                    LA4_39 = self.input.LA(8)

                                    if (LA4_39 == 84) :
                                        LA4_41 = self.input.LA(9)

                                        if (LA4_41 == 73) :
                                            LA4_42 = self.input.LA(10)

                                            if (LA4_42 == 86) :
                                                LA4_43 = self.input.LA(11)

                                                if (LA4_43 == 73) :
                                                    LA4_44 = self.input.LA(12)

                                                    if (LA4_44 == 84) :
                                                        LA4_45 = self.input.LA(13)

                                                        if (LA4_45 == 89) :
                                                            LA4_46 = self.input.LA(14)

                                                            if ((48 <= LA4_46 <= 57) or (65 <= LA4_46 <= 90) or LA4_46 == 95 or (97 <= LA4_46 <= 122)) :
                                                                alt4 = 16
                                                            else:
                                                                alt4 = 5

                                                        else:
                                                            alt4 = 16

                                                    else:
                                                        alt4 = 16

                                                else:
                                                    alt4 = 16

                                            else:
                                                alt4 = 16

                                        else:
                                            alt4 = 16

                                    else:
                                        alt4 = 16

                                else:
                                    alt4 = 16

                            else:
                                alt4 = 16

                        else:
                            alt4 = 16

                    else:
                        alt4 = 16

                else:
                    alt4 = 16

            else:
                alt4 = 16

        elif LA4 == 105:
            LA4_6 = self.input.LA(2)

            if (LA4_6 == 110) :
                LA4_20 = self.input.LA(3)

                if ((48 <= LA4_20 <= 57) or (65 <= LA4_20 <= 90) or LA4_20 == 95 or (97 <= LA4_20 <= 122)) :
                    alt4 = 16
                else:
                    alt4 = 6

            else:
                alt4 = 16

        elif LA4 == 40:
            alt4 = 7
        elif LA4 == 41:
            alt4 = 8
        elif LA4 == 91:
            alt4 = 9
        elif LA4 == 93:
            alt4 = 10
        elif LA4 == 59:
            alt4 = 11
        elif LA4 == 44:
            alt4 = 12
        elif LA4 == 97:
            LA4_13 = self.input.LA(2)

            if (LA4_13 == 110) :
                LA4_21 = self.input.LA(3)

                if (LA4_21 == 100) :
                    LA4_27 = self.input.LA(4)

                    if ((48 <= LA4_27 <= 57) or (65 <= LA4_27 <= 90) or LA4_27 == 95 or (97 <= LA4_27 <= 122)) :
                        alt4 = 16
                    else:
                        alt4 = 13

                else:
                    alt4 = 16

            else:
                alt4 = 16

        elif LA4 == 111:
            LA4_14 = self.input.LA(2)

            if (LA4_14 == 114) :
                LA4_22 = self.input.LA(3)

                if ((48 <= LA4_22 <= 57) or (65 <= LA4_22 <= 90) or LA4_22 == 95 or (97 <= LA4_22 <= 122)) :
                    alt4 = 16
                else:
                    alt4 = 14

            else:
                alt4 = 16

        elif LA4 == 110:
            LA4_15 = self.input.LA(2)

            if (LA4_15 == 111) :
                LA4_23 = self.input.LA(3)

                if (LA4_23 == 116) :
                    LA4_29 = self.input.LA(4)

                    if ((48 <= LA4_29 <= 57) or (65 <= LA4_29 <= 90) or LA4_29 == 95 or (97 <= LA4_29 <= 122)) :
                        alt4 = 16
                    else:
                        alt4 = 15

                else:
                    alt4 = 16

            else:
                alt4 = 16

        elif LA4 == 65 or LA4 == 66 or LA4 == 68 or LA4 == 69 or LA4 == 70 or LA4 == 71 or LA4 == 72 or LA4 == 73 or LA4 == 74 or LA4 == 75 or LA4 == 76 or LA4 == 77 or LA4 == 78 or LA4 == 79 or LA4 == 80 or LA4 == 81 or LA4 == 82 or LA4 == 83 or LA4 == 84 or LA4 == 85 or LA4 == 86 or LA4 == 87 or LA4 == 88 or LA4 == 89 or LA4 == 90 or LA4 == 95 or LA4 == 98 or LA4 == 99 or LA4 == 100 or LA4 == 101 or LA4 == 102 or LA4 == 103 or LA4 == 104 or LA4 == 106 or LA4 == 107 or LA4 == 108 or LA4 == 109 or LA4 == 113 or LA4 == 114 or LA4 == 115 or LA4 == 116 or LA4 == 117 or LA4 == 118 or LA4 == 119 or LA4 == 120 or LA4 == 121 or LA4 == 122:
            alt4 = 16
        elif LA4 == 48 or LA4 == 49 or LA4 == 50 or LA4 == 51 or LA4 == 52 or LA4 == 53 or LA4 == 54 or LA4 == 55 or LA4 == 56 or LA4 == 57:
            alt4 = 17
        else:
            nvae = NoViableAltException("", 4, 0, self.input)

            raise nvae


        if alt4 == 1:
            # cadbiom/models/guard_transitions/translators/pintlang.g:1:10: WS
            pass 
            self.mWS()



        elif alt4 == 2:
            # cadbiom/models/guard_transitions/translators/pintlang.g:1:13: COMMENT
            pass 
            self.mCOMMENT()



        elif alt4 == 3:
            # cadbiom/models/guard_transitions/translators/pintlang.g:1:21: PROC
            pass 
            self.mPROC()



        elif alt4 == 4:
            # cadbiom/models/guard_transitions/translators/pintlang.g:1:26: ARROW
            pass 
            self.mARROW()



        elif alt4 == 5:
            # cadbiom/models/guard_transitions/translators/pintlang.g:1:32: COOP
            pass 
            self.mCOOP()



        elif alt4 == 6:
            # cadbiom/models/guard_transitions/translators/pintlang.g:1:37: IN_KW
            pass 
            self.mIN_KW()



        elif alt4 == 7:
            # cadbiom/models/guard_transitions/translators/pintlang.g:1:43: LP
            pass 
            self.mLP()



        elif alt4 == 8:
            # cadbiom/models/guard_transitions/translators/pintlang.g:1:46: RP
            pass 
            self.mRP()



        elif alt4 == 9:
            # cadbiom/models/guard_transitions/translators/pintlang.g:1:49: LB
            pass 
            self.mLB()



        elif alt4 == 10:
            # cadbiom/models/guard_transitions/translators/pintlang.g:1:52: RB
            pass 
            self.mRB()



        elif alt4 == 11:
            # cadbiom/models/guard_transitions/translators/pintlang.g:1:55: SC
            pass 
            self.mSC()



        elif alt4 == 12:
            # cadbiom/models/guard_transitions/translators/pintlang.g:1:58: COM
            pass 
            self.mCOM()



        elif alt4 == 13:
            # cadbiom/models/guard_transitions/translators/pintlang.g:1:62: AND
            pass 
            self.mAND()



        elif alt4 == 14:
            # cadbiom/models/guard_transitions/translators/pintlang.g:1:66: OR
            pass 
            self.mOR()



        elif alt4 == 15:
            # cadbiom/models/guard_transitions/translators/pintlang.g:1:69: NOT
            pass 
            self.mNOT()



        elif alt4 == 16:
            # cadbiom/models/guard_transitions/translators/pintlang.g:1:73: IDENT
            pass 
            self.mIDENT()



        elif alt4 == 17:
            # cadbiom/models/guard_transitions/translators/pintlang.g:1:79: INT
            pass 
            self.mINT()








 



def main(argv, stdin=sys.stdin, stdout=sys.stdout, stderr=sys.stderr):
    from antlr3.main import LexerMain
    main = LexerMain(pintlangLexer)

    main.stdin = stdin
    main.stdout = stdout
    main.stderr = stderr
    main.execute(argv)



if __name__ == '__main__':
    main(sys.argv)
